# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""Transformer.

This script defines a serving transformer which can preprocess raw data
and postprocess the predictions.
"""

import joblib
import kserve

from kale.serve import utils
from typing import Dict


class_names = ['alt.atheism', 'comp.graphics',
               'comp.os.ms-windows.misc',
               'comp.sys.ibm.pc.hardware',
               'comp.sys.mac.hardware',
               'comp.windows.x', 'misc.forsale',
               'rec.autos', 'rec.motorcycles',
               'rec.sport.baseball', 'rec.sport.hockey',
               'sci.crypt', 'sci.electronics', 'sci.med',
               'sci.space', 'soc.religion.christian',
               'talk.politics.guns', 'talk.politics.mideast',
               'talk.politics.misc', 'talk.religion.misc']


class Transformer(kserve.Model):
    """Transform the data.

    Vectorize the input data before passing it to the
    model and return human-readable predictions.

    Args:
        name (str): The name of the Transformer
        predictor_host (str): The host address of the Predictor
    """

    def __init__(self, model_name: str, predictor_host: str,
                 protocol: str = "v1"):
        super().__init__(model_name)
        self.predictor_host = predictor_host
        self.protocol = protocol

        # load the vectorizer object
        path = utils.get_transformer_asset("vectorizer.joblib")
        with open(path, "rb") as f:
            self.vectorizer = joblib.load(f)

    def preprocess(self, inputs: Dict):
        """Preprocess the dataset."""
        transformed_data = self.vectorizer.transform(inputs["instances"])
        return {'instances': transformed_data.toarray().tolist()}

    def postprocess(self, inputs: Dict):
        """Postprocess the predictions."""
        return {"predictions": [class_names[i] for i in inputs["predictions"]]}
